@@ -3,9 +3,6 @@ require 'active_support/core_ext/class/attribute'
 module RablRails
   module Handlers
     class Rabl
-      class_attribute :default_format
-      self.default_format = 'application/json'
-
       def self.call(template)
         %{
           RablRails::Library.instance.
